@@ -28,7 +28,7 @@
 
 using System;
 
-using Banshee.Base;
+using Hyena;
 
 namespace Banshee.IO
 {
@@ -40,7 +40,7 @@ namespace Banshee.IO
         void Copy (SafeUri from, SafeUri to, bool overwrite);
         long GetSize (SafeUri uri);
         long GetModifiedTime (SafeUri uri);
-        
+
         System.IO.Stream OpenRead (SafeUri uri);
         System.IO.Stream OpenWrite (SafeUri uri, bool overwrite);
     }
