@@ -1,4 +1,4 @@
-// 
+//
 // DatabaseImportManager.cs
 //
 // Authors:
@@ -40,6 +40,7 @@ using Banshee.Base;
 using Banshee.Sources;
 using Banshee.Collection;
 using Banshee.Collection.Database;
+using Banshee.Configuration.Schema;
 using Banshee.ServiceStack;
 using Banshee.Streaming;
 
@@ -51,40 +52,27 @@ namespace Banshee.Collection.Database
         // in this list do not mean they are actually supported - this list is just
         // used to see if we should allow the file to be processed by TagLib. The
         // point is to rule out, at the path level, files that we won't support.
-        
-        public static readonly string [] WhiteListFileExtensions = new string [] {
-            "3g2",  "3gp",  "3gp2", "3gpp", "aac",  "ac3",  "aif",  "aifc", 
-            "aiff", "al",   "alaw", "ape",  "asf",  "asx",  "au",   "avi", 
-            "cda",  "cdr",  "divx", "dv",   "flac", "flv",  "gvi",  "gvp", 
-            "m1v",  "m21",  "m2p",  "m2v",  "m4a",  "m4b",  "m4e",  "m4p",  
+
+        public static readonly Banshee.IO.ExtensionSet WhiteListFileExtensions = new Banshee.IO.ExtensionSet (
+            "3g2",  "3gp",  "3gp2", "3gpp", "aac",  "ac3",  "aif",  "aifc",
+            "aiff", "al",   "alaw", "ape",  "asf",  "asx",  "au",   "avi",
+            "cda",  "cdr",  "divx", "dv",   "flac", "flv",  "gvi",  "gvp",
+            "m1v",  "m21",  "m2p",  "m2v",  "m4a",  "m4b",  "m4e",  "m4p",
             "m4u",  "m4v",  "mp+",  "mid",  "midi", "mjp",  "mkv",  "moov",
             "mov",  "movie","mp1",  "mp2",  "mp21", "mp3",  "mp4",  "mpa",
-            "mpc",  "mpe",  "mpeg", "mpg",  "mpga", "mpp",  "mpu",  "mpv",  
+            "mpc",  "mpe",  "mpeg", "mpg",  "mpga", "mpp",  "mpu",  "mpv",
             "mpv2", "oga",  "ogg",  "ogv",  "ogm",  "omf",  "qt",   "ra",
-            "ram",  "raw",  "rm",   "rmvb", "rts",  "smil", "swf",  "tivo", 
-            "u",    "vfw",  "vob",  "wav",  "wave", "wax",  "wm",   "wma",  
-            "wmd",  "wmv",  "wmx",  "wv",   "wvc",  "wvx",  "yuv",  "f4v",  
-            "f4a",  "f4b"
-        };
-
-        static DatabaseImportManager ()
-        {
-            Array.Sort<string> (WhiteListFileExtensions);
-        }
+            "ram",  "raw",  "rm",   "rmvb", "rts",  "smil", "swf",  "tivo",
+            "u",    "vfw",  "vob",  "wav",  "wave", "wax",  "wm",   "wma",
+            "wmd",  "wmv",  "wmx",  "wv",   "wvc",  "wvx",  "yuv",  "f4v",
+            "f4a",  "f4b",  "669",  "it",   "med",  "mod",  "mol",  "mtm",
+            "nst",  "s3m",  "stm",  "ult",  "wow",  "xm",   "xnm",  "spx",
+            "ts"
+        );
 
         public static bool IsWhiteListedFile (string path)
         {
-            if (String.IsNullOrEmpty (path)) {
-                return false;
-            }
-            
-            int index = path.LastIndexOf ('.');
-            if (index < 0 || index == path.Length || index == path.Length - 1) {
-                return false;
-            }
-            
-            return Array.BinarySearch<string> (WhiteListFileExtensions, 
-                path.Substring (index + 1).ToLower ()) >= 0;
+            return WhiteListFileExtensions.IsMatchingFile (path);
         }
 
         public delegate PrimarySource TrackPrimarySourceChooser (DatabaseTrackInfo track);
@@ -95,15 +83,15 @@ namespace Banshee.Collection.Database
         private int [] primary_source_ids;
         private string base_directory;
         private bool force_copy;
-        
+
         public event DatabaseImportResultHandler ImportResult;
-    
+
         public DatabaseImportManager (PrimarySource psource) :
             this (psource.ErrorSource, delegate { return psource; }, new int [] {psource.DbId}, psource.BaseDirectory)
         {
         }
 
-        public DatabaseImportManager (ErrorSource error_source, TrackPrimarySourceChooser chooser, 
+        public DatabaseImportManager (ErrorSource error_source, TrackPrimarySourceChooser chooser,
             int [] primarySourceIds, string baseDirectory) : this (chooser)
         {
             this.error_source = error_source;
@@ -130,7 +118,7 @@ namespace Banshee.Collection.Database
             get { return base_directory; }
             set { base_directory = value; }
         }
-        
+
         protected virtual bool ForceCopy {
             get { return force_copy; }
             set { force_copy = value; }
@@ -138,18 +126,15 @@ namespace Banshee.Collection.Database
 
         protected override void OnImportRequested (string path)
         {
-            if (!IsWhiteListedFile (path)) {
-                UpdateProgress (null);
-                return;
-            }
-
             try {
                 DatabaseTrackInfo track = ImportTrack (path);
                 if (track != null && track.TrackId > 0) {
-                    UpdateProgress (String.Format ("{0} - {1}", 
+                    UpdateProgress (String.Format ("{0} - {1}",
                         track.DisplayArtistName, track.DisplayTrackTitle));
+                } else {
+                    UpdateProgress (null);
                 }
-                
+
                 OnImportResult (track, path, null);
             } catch (Exception e) {
                 LogError (path, e);
@@ -157,7 +142,7 @@ namespace Banshee.Collection.Database
                 OnImportResult (null, path, e);
             }
         }
-        
+
         protected virtual void OnImportResult (DatabaseTrackInfo track, string path, Exception error)
         {
             DatabaseImportResultHandler handler = ImportResult;
@@ -165,7 +150,7 @@ namespace Banshee.Collection.Database
                 handler (this, new DatabaseImportResultArgs (track, path, error));
             }
         }
-        
+
         public DatabaseTrackInfo ImportTrack (string path)
         {
             return ImportTrack (new SafeUri (path));
@@ -173,40 +158,47 @@ namespace Banshee.Collection.Database
 
         public DatabaseTrackInfo ImportTrack (SafeUri uri)
         {
-            DatabaseTrackInfo track = null;
-            
-            if (DatabaseTrackInfo.ContainsUri (uri, Paths.MakePathRelative (uri.AbsolutePath, BaseDirectory) ?? uri.AbsoluteUri, PrimarySourceIds)) {
-                // TODO add DatabaseTrackInfo.SyncedStamp property, and if the file has been
-                // updated since the last sync, fetch its metadata into the db.
-                UpdateProgress (null);
+            if (!IsWhiteListedFile (uri.AbsoluteUri)) {
                 return null;
             }
 
+            if (DatabaseTrackInfo.ContainsUri (uri, PrimarySourceIds)) {
+                // TODO add DatabaseTrackInfo.SyncedStamp property, and if the file has been
+                // updated since the last sync, fetch its metadata into the db.
+                return null;
+            }
+
+            DatabaseTrackInfo track = null;
+
             // TODO note, there is deadlock potential here b/c of locking of shared commands and blocking
             // because of transactions.  Needs to be fixed in HyenaDatabaseConnection.
-            //ServiceManager.DbConnection.BeginTransaction ();
+            ServiceManager.DbConnection.BeginTransaction ();
             try {
-                TagLib.File file = StreamTagger.ProcessUri (uri);
                 track = new DatabaseTrackInfo ();
-                StreamTagger.TrackInfoMerge (track, file);
-                
-                track.DateAdded = DateTime.Now;
-                track.PrimarySource = trackPrimarySourceChooser (track);
-
-                if (track.PrimarySource is Banshee.Library.LibrarySource) {
-                    track.CopyToLibraryIfAppropriate (force_copy);
+                track.Uri = uri;
+                using (var file = StreamTagger.ProcessUri (uri)) {
+                    StreamTagger.TrackInfoMerge (track, file, false, true);
                 }
 
-                track.Save (false);
+                track.PrimarySource = trackPrimarySourceChooser (track);
 
-                //ServiceManager.DbConnection.CommitTransaction ();
+                bool save_track = true;
+                if (track.PrimarySource is Banshee.Library.LibrarySource) {
+                    save_track = track.CopyToLibraryIfAppropriate (force_copy);
+                }
+
+                if (save_track) {
+                    track.Save (false);
+                }
+
+                ServiceManager.DbConnection.CommitTransaction ();
             } catch (Exception) {
-                //ServiceManager.DbConnection.RollbackTransaction ();
+                ServiceManager.DbConnection.RollbackTransaction ();
                 throw;
             }
 
             counts[track.PrimarySourceId] = counts.ContainsKey (track.PrimarySourceId) ? counts[track.PrimarySourceId] + 1 : 1;
-            
+
             // Reload every 20% or every 250 tracks, whatever is more (eg at most reload 5 times during an import)
             if (counts[track.PrimarySourceId] >= Math.Max (TotalCount/5, 250)) {
                 counts[track.PrimarySourceId] = 0;
@@ -227,10 +219,10 @@ namespace Banshee.Collection.Database
 
         private void LogError (string path, string msg)
         {
-            ErrorSource.AddMessage (Path.GetFileName (path), msg);
+            ErrorSource.AddMessage (path, msg);
             Log.Error (path, msg, false);
         }
-        
+
         public void NotifyAllSources ()
         {
             foreach (int primary_source_id in counts.Keys) {
@@ -239,10 +231,10 @@ namespace Banshee.Collection.Database
             counts.Clear ();
         }
 
-        protected override void OnImportFinished ()
+        protected override void OnFinished ()
         {
             NotifyAllSources ();
-            base.OnImportFinished ();
+            base.OnFinished ();
         }
     }
 }
