@@ -38,6 +38,7 @@ using Hyena.Data.Sqlite;
 using Hyena.Collections;
 
 using Banshee.Base;
+using Banshee.Preferences;
 using Banshee.ServiceStack;
 using Banshee.Configuration;
 using Banshee.Sources;
@@ -79,27 +80,14 @@ namespace Banshee.Sources
     public abstract class PrimarySource : DatabaseSource, IDisposable
     {
         #region Functions that let us override some behavior of our DatabaseTrackInfos
-        
-        private TrackEqualHandler track_equal_handler;
-        public TrackEqualHandler TrackEqualHandler {
-            get { return track_equal_handler; }
-            protected set { track_equal_handler = value; }
-        }
 
-        private TrackExternalObjectHandler track_external_object_handler;
-        public TrackExternalObjectHandler TrackExternalObjectHandler {
-            get { return track_external_object_handler; }
-            protected set { track_external_object_handler = value; }
-        }
-
-        private TrackArtworkIdHandler track_artwork_id_handler;
-        public TrackArtworkIdHandler TrackArtworkIdHandler {
-            get { return track_artwork_id_handler; }
-            protected set { track_artwork_id_handler = value; }
-        }
+        public TrackEqualHandler TrackEqualHandler { get; protected set; }
+        public TrackInfo.IsPlayingHandler TrackIsPlayingHandler { get; protected set; }
+        public TrackExternalObjectHandler TrackExternalObjectHandler { get; protected set; }
+        public TrackArtworkIdHandler TrackArtworkIdHandler { get; protected set; }
 
         #endregion
-    
+
         protected ErrorSource error_source;
         protected bool error_source_visible = false;
 
@@ -116,7 +104,7 @@ namespace Banshee.Sources
             DELETE FROM CoreArtists WHERE ArtistID NOT IN (SELECT ArtistID FROM CoreTracks);
             DELETE FROM CoreAlbums WHERE AlbumID NOT IN (SELECT AlbumID FROM CoreTracks)
         ");
-        
+
         protected HyenaSqliteCommand purge_tracks_command = new HyenaSqliteCommand (@"
             DELETE FROM CoreTracks WHERE PrimarySourceId = ?
         ");
@@ -132,18 +120,18 @@ namespace Banshee.Sources
                 if (dbid > 0) {
                     return dbid;
                 }
-                
+
                 dbid = ServiceManager.DbConnection.Query<int> ("SELECT PrimarySourceID FROM CorePrimarySources WHERE StringID = ?", UniqueId);
                 if (dbid == 0) {
                     dbid = ServiceManager.DbConnection.Execute ("INSERT INTO CorePrimarySources (StringID) VALUES (?)", UniqueId);
                 } else {
                     SavedCount = ServiceManager.DbConnection.Query<int> ("SELECT CachedCount FROM CorePrimarySources WHERE PrimarySourceID = ?", dbid);
                 }
-                
+
                 if (dbid == 0) {
                     throw new ApplicationException ("dbid could not be resolved, this should never happen");
                 }
-                
+
                 return dbid;
             }
         }
@@ -175,17 +163,17 @@ namespace Banshee.Sources
             get { return is_local; }
             protected set { is_local = value; }
         }
-        
+
         private static SourceSortType[] sort_types = new SourceSortType[] {
             SortNameAscending,
             SortSizeAscending,
             SortSizeDescending
         };
-        
+
         public override SourceSortType[] ChildSortTypes {
             get { return sort_types; }
         }
-        
+
         public override SourceSortType DefaultChildSort {
             get { return SortNameAscending; }
         }
@@ -206,7 +194,7 @@ namespace Banshee.Sources
             get { return null; }
             protected set { base_dir_with_sep = null; }
         }
-        
+
         private string base_dir_with_sep;
         public string BaseDirectoryWithSeparator {
             get { return base_dir_with_sep ?? (base_dir_with_sep = BaseDirectory + System.IO.Path.DirectorySeparatorChar); }
@@ -214,7 +202,7 @@ namespace Banshee.Sources
 
         protected PrimarySource (string generic_name, string name, string id, int order) : base (generic_name, name, id, order)
         {
-            Properties.SetString ("SortChildrenActionLabel", Catalog.GetString ("Sort Playlists by"));
+            Properties.SetString ("SortChildrenActionLabel", Catalog.GetString ("Sort Playlists By"));
             PrimarySourceInitialize ();
         }
 
@@ -235,7 +223,21 @@ namespace Banshee.Sources
 
         public override bool Expanded {
             get { return ExpandedSchema.Get (); }
-            set { ExpandedSchema.Set (value); }
+            set {
+                ExpandedSchema.Set (value);
+                base.Expanded = value;
+            }
+        }
+
+        public PathPattern PathPattern { get; private set; }
+
+        protected void SetFileNamePattern (PathPattern pattern)
+        {
+            PathPattern = pattern;
+
+            var file_system = PreferencesPage.Add (new Section ("file-system", Catalog.GetString ("File Organization"), 5));
+            file_system.Add (new SchemaPreference<string> (pattern.FolderSchema, Catalog.GetString ("Folder hie_rarchy")));
+            file_system.Add (new SchemaPreference<string> (pattern.FileSchema, Catalog.GetString ("File _name")));
         }
 
         public virtual void Dispose ()
@@ -264,7 +266,7 @@ namespace Banshee.Sources
             DatabaseTrackModel.AddCondition (String.Format ("CoreTracks.PrimarySourceID = {0}", DbId));
 
             primary_sources[DbId] = this;
-            
+
             // Load our playlists and smart playlists
             foreach (PlaylistSource pl in PlaylistSource.LoadAll (this)) {
                 AddChildSource (pl);
@@ -335,7 +337,7 @@ namespace Banshee.Sources
             OnTracksChanged ();
         }
 
-        internal void NotifyTracksDeleted ()
+        public void NotifyTracksDeleted ()
         {
             OnTracksDeleted ();
         }
@@ -375,15 +377,6 @@ namespace Banshee.Sources
             get { return true; }
         }
 
-        public override void SetParentSource (Source source)
-        {
-            if (source is PrimarySource) {
-                throw new ArgumentException ("PrimarySource cannot have another PrimarySource as its parent");
-            }
-
-            base.SetParentSource (source);
-        }
-
         protected override void OnTracksAdded ()
         {
             ThreadAssist.SpawnFromMain (delegate {
@@ -431,7 +424,7 @@ namespace Banshee.Sources
         {
             OnTracksDeleted ();
         }
-        
+
         protected virtual void PurgeTracks ()
         {
             ServiceManager.DbConnection.Execute (purge_tracks_command, DbId);
@@ -464,7 +457,7 @@ namespace Banshee.Sources
 
             if (source.Count < 1)
                 return;
-            
+
             ThreadAssist.SpawnFromMain (delegate {
                 CachedList<DatabaseTrackInfo> list = CachedList<DatabaseTrackInfo>.CreateFromModel (source.DatabaseTrackModel);
                 DeleteTrackList (list);
@@ -476,7 +469,7 @@ namespace Banshee.Sources
             if (model == null || model.Count < 1) {
                 return;
             }
-            
+
             ThreadAssist.SpawnFromMain (delegate {
                 CachedList<DatabaseTrackInfo> list = CachedList<DatabaseTrackInfo>.CreateFromModelSelection (model);
                 DeleteTrackList (list);
@@ -521,7 +514,7 @@ namespace Banshee.Sources
                 delete_track_job.Finish ();
                 delete_track_job = null;
             }
-            
+
             if (skip_deletion != null) {
                 list.Remove (skip_deletion);
                 skip_deletion.Clear ();
@@ -532,7 +525,7 @@ namespace Banshee.Sources
             if (list.Count > 0) {
                 ServiceManager.DbConnection.Execute (remove_list_command, DateTime.Now, list.CacheId, list.CacheId);
             }
-            
+
             ThreadAssist.ProxyToMain (delegate {
                 OnTracksDeleted ();
                 OnUserNotifyUpdated ();
@@ -542,7 +535,16 @@ namespace Banshee.Sources
 
         protected virtual bool DeleteTrack (DatabaseTrackInfo track)
         {
-            throw new Exception ("PrimarySource DeleteTrack method not implemented");
+            if (!track.Uri.IsLocalPath)
+                throw new Exception ("Cannot delete a non-local resource: " + track.Uri.Scheme);
+
+            try {
+                Banshee.IO.Utilities.DeleteFileTrimmingParentDirectories (track.Uri);
+            } catch (System.IO.FileNotFoundException) {
+            } catch (System.IO.DirectoryNotFoundException) {
+            }
+
+            return true;
         }
 
         public override bool AcceptsInputFromSource (Source source)
@@ -568,13 +570,13 @@ namespace Banshee.Sources
             }
             return true;
         }
-        
+
         public override bool AddAllTracks (Source source)
         {
             if (!AcceptsInputFromSource (source) || source.Count == 0) {
                 return false;
             }
-            
+
             DatabaseTrackListModel model = (source as ITrackModelSource).TrackModel as DatabaseTrackListModel;
             CachedList<DatabaseTrackInfo> cached_list = CachedList<DatabaseTrackInfo>.CreateFromModel (model);
             if (ThreadAssist.InMainThread) {
@@ -620,7 +622,7 @@ namespace Banshee.Sources
                     IncrementAddedTracks ();
                     break;
                 }
-                
+
                 if (track == null) {
                     IncrementAddedTracks ();
                     continue;
@@ -661,7 +663,7 @@ namespace Banshee.Sources
             if (notify || finished) {
                 OnTracksAdded ();
                 if (finished) {
-                    Banshee.Base.ThreadAssist.ProxyToMain (OnUserNotifyUpdated);
+                    ThreadAssist.ProxyToMain (OnUserNotifyUpdated);
                 }
             }
         }
@@ -684,7 +686,7 @@ namespace Banshee.Sources
                 lock (this) {
                     if (add_track_job == null) {
                         add_track_job = new BatchUserJob (String.Format (Catalog.GetString (
-                            "Adding {0} of {1} to {2}"), "{0}", "{1}", Name), 
+                            "Adding {0} of {1} to {2}"), "{0}", "{1}", Name),
                             Properties.GetStringList ("Icon.Name"));
                         add_track_job.SetResources (Resource.Cpu, Resource.Database, Resource.Disk);
                         add_track_job.PriorityHints = PriorityHints.SpeedSensitive | PriorityHints.DataLossIfStopped;
