@@ -55,6 +55,11 @@ namespace Banshee.Sources
         private List<Source> child_sources = new List<Source> ();
         private ReadOnlyCollection<Source> read_only_children;
 
+        private SourceSortType child_sort;
+        private bool sort_children = true;
+        private SchemaEntry<string> child_sort_schema;
+        private SchemaEntry<bool> separate_by_type_schema;
+
         public event EventHandler Updated;
         public event EventHandler UserNotifyUpdated;
         public event EventHandler MessageNotify;
@@ -67,7 +72,7 @@ namespace Banshee.Sources
         {
         }
 
-        protected Source (string generic_name, string name, int order, string type_unique_id)
+        protected Source (string generic_name, string name, int order, string type_unique_id) : this ()
         {
             GenericName = generic_name;
             Name = name;
@@ -79,6 +84,7 @@ namespace Banshee.Sources
 
         protected Source ()
         {
+            child_sort = DefaultChildSort;
         }
 
         // This method is chained to subclasses intialize methods,
@@ -99,19 +105,21 @@ namespace Banshee.Sources
 
             properties.PropertyChanged += OnPropertyChanged;
             read_only_children = new ReadOnlyCollection<Source> (child_sources);
-            
+
             if (ApplicationContext.Debugging && ApplicationContext.CommandLine.Contains ("test-source-messages")) {
                 TestMessages ();
             }
+
+            LoadSortSchema ();
         }
-        
+
         protected void OnSetupComplete ()
         {
             /*ITrackModelSource tm_source = this as ITrackModelSource;
             if (tm_source != null) {
                 tm_source.TrackModel.Parent = this;
                 ServiceManager.DBusServiceManager.RegisterObject (tm_source.TrackModel);
-                
+
                 // TODO if/when browsable models can be added/removed on the fly, this would need to change to reflect that
                 foreach (IListModel model in tm_source.FilterModels) {
                     Banshee.Collection.ExportableModel exportable = model as Banshee.Collection.ExportableModel;
@@ -125,6 +133,10 @@ namespace Banshee.Sources
 
         protected void Remove ()
         {
+            if (prefs_page != null) {
+                prefs_page.Dispose ();
+            }
+
             if (ServiceManager.SourceManager.ContainsSource (this)) {
                 if (this.Parent != null) {
                     this.Parent.RemoveChildSource (this);
@@ -134,8 +146,18 @@ namespace Banshee.Sources
             }
         }
 
+        protected void PauseSorting ()
+        {
+            sort_children = false;
+        }
+
+        protected void ResumeSorting ()
+        {
+            sort_children = true;
+        }
+
 #region Public Methods
-        
+
         public virtual void Activate ()
         {
         }
@@ -148,26 +170,38 @@ namespace Banshee.Sources
         {
             properties.SetString ("Name", newName);
         }
-        
+
         public virtual bool AcceptsInputFromSource (Source source)
         {
             return false;
         }
-        
+
+        public virtual bool AcceptsUserInputFromSource (Source source)
+        {
+            return AcceptsInputFromSource (source);
+        }
+
         public virtual void MergeSourceInput (Source source, SourceMergeType mergeType)
         {
             Log.ErrorFormat ("MergeSourceInput not implemented by {0}", this);
         }
-        
+
         public virtual SourceMergeType SupportedMergeTypes {
             get { return SourceMergeType.None; }
         }
-        
+
         public virtual void SetParentSource (Source parent)
         {
             this.parent = parent;
         }
-        
+
+        public virtual bool ContainsChildSource (Source child)
+        {
+            lock (Children) {
+                return child_sources.Contains (child);
+            }
+        }
+
         public virtual void AddChildSource (Source child)
         {
             lock (Children) {
@@ -185,19 +219,12 @@ namespace Banshee.Sources
                 if (child.Children.Count > 0) {
                     child.ClearChildSources ();
                 }
-                
+
                 child_sources.Remove (child);
-                
-                if (ServiceManager.SourceManager.ActiveSource == child) {
-                    if (CanActivate) {
-                        ServiceManager.SourceManager.SetActiveSource (this);
-                    }
-                }
-                
                 OnChildSourceRemoved (child);
             }
         }
-        
+
         public virtual void ClearChildSources ()
         {
             lock (Children) {
@@ -207,15 +234,7 @@ namespace Banshee.Sources
             }
         }
 
-        public class NameComparer : IComparer<Source>
-        {
-            public int Compare (Source a, Source b)
-            {
-                return a.Name.CompareTo (b.Name);
-            }
-        }
-
-        public class SizeComparer : IComparer<Source>
+        private class SizeComparer : IComparer<Source>
         {
             public int Compare (Source a, Source b)
             {
@@ -223,34 +242,81 @@ namespace Banshee.Sources
             }
         }
 
-        public virtual void SortChildSources (IComparer<Source> comparer, bool asc)
+        public virtual void SortChildSources (SourceSortType sort_type)
         {
-            lock (Children) {
-                child_sources.Sort (comparer);
-                if (!asc) {
-                    child_sources.Reverse ();
-                }
+            child_sort = sort_type;
+            child_sort_schema.Set (child_sort.Id);
+            SortChildSources ();
+        }
 
-                int i = 0;
-                foreach (Source child in child_sources) {
-                    child.Order = i++;
+        public virtual void SortChildSources ()
+        {
+            lock (this) {
+                if (!sort_children) {
+                    return;
+                }
+                sort_children = false;
+            }
+
+            if (child_sort != null && child_sort.SortType != SortType.None) {
+                lock (Children) {
+                    child_sort.Sort (child_sources, SeparateChildrenByType);
+
+                    int i = 0;
+                    foreach (Source child in child_sources) {
+                        // Leave children with negative orders alone, so they can be manually
+                        // placed at the top
+                        if (child.Order >= 0) {
+                            child.Order = i++;
+                        }
+                    }
                 }
             }
+            sort_children = true;
         }
-        
+
+        private void LoadSortSchema ()
+        {
+            if (ChildSortTypes.Length == 0) {
+                return;
+            }
+
+            if (unique_id == null && type_unique_id == null) {
+                Hyena.Log.WarningFormat ("Trying to LoadSortSchema, but source's id not set! {0}", UniqueId);
+                return;
+            }
+
+            child_sort_schema = CreateSchema<string> ("child_sort_id", DefaultChildSort.Id, "", "");
+            string child_sort_id = child_sort_schema.Get ();
+            foreach (SourceSortType sort_type in ChildSortTypes) {
+                if (sort_type.Id == child_sort_id) {
+                    child_sort = sort_type;
+                    break;
+                }
+            }
+
+            separate_by_type_schema = CreateSchema<bool> ("separate_by_type", false, "", "");
+            SortChildSources ();
+        }
+
+        public T GetProperty<T> (string name, bool inherited)
+        {
+            return inherited ? GetInheritedProperty<T> (name) : Properties.Get<T> (name);
+        }
+
         public T GetInheritedProperty<T> (string name)
         {
             return Properties.Contains (name)
                 ? Properties.Get<T> (name)
-                : Parent != null 
+                : Parent != null
                     ? Parent.GetInheritedProperty<T> (name)
                     : default (T);
         }
-        
+
 #endregion
-        
+
 #region Protected Methods
-        
+
         public virtual void SetStatus (string message, bool error)
         {
             SetStatus (message, !error, !error, error ? "dialog-error" : null);
@@ -263,17 +329,18 @@ namespace Banshee.Sources
                     status_message = new SourceMessage (this);
                     PushMessage (status_message);
                 }
-            
+
                 string status_name = String.Format ("<i>{0}</i>", GLib.Markup.EscapeText (Name));
-                
+
                 status_message.FreezeNotify ();
                 status_message.Text = String.Format (GLib.Markup.EscapeText (message), status_name);
                 status_message.CanClose = can_close;
                 status_message.IsSpinning = is_spinning;
                 status_message.SetIconName (icon_name);
+                status_message.IsHidden = false;
                 status_message.ClearActions ();
             }
-                
+
             status_message.ThawNotify ();
         }
 
@@ -287,18 +354,17 @@ namespace Banshee.Sources
             }
         }
 
-
-        protected virtual void PushMessage (SourceMessage message)
+        public void PushMessage (SourceMessage message)
         {
             lock (this) {
                 messages.Insert (0, message);
                 message.Updated += HandleMessageUpdated;
             }
-            
+
             OnMessageNotify ();
         }
-        
-        protected virtual SourceMessage PopMessage ()
+
+        protected SourceMessage PopMessage ()
         {
             try {
                 lock (this) {
@@ -308,15 +374,15 @@ namespace Banshee.Sources
                         messages.RemoveAt (0);
                         return message;
                     }
-                    
+
                     return null;
                 }
             } finally {
                 OnMessageNotify ();
             }
         }
-        
-        protected virtual void ClearMessages ()
+
+        protected void ClearMessages ()
         {
             lock (this) {
                 if (messages.Count > 0) {
@@ -326,14 +392,16 @@ namespace Banshee.Sources
                     messages.Clear ();
                     OnMessageNotify ();
                 }
+
+                status_message = null;
             }
         }
-        
+
         private void TestMessages ()
         {
             int count = 0;
             SourceMessage message_3 = null;
-            
+
             Application.RunTimeout (5000, delegate {
                 if (count++ > 5) {
                     if (count == 7) {
@@ -344,7 +412,7 @@ namespace Banshee.Sources
                 } else if (count > 10) {
                     return false;
                 }
-                
+
                 SourceMessage message = new SourceMessage (this);
                 message.FreezeNotify ();
                 message.Text = String.Format ("Testing message {0}", count);
@@ -355,33 +423,37 @@ namespace Banshee.Sources
                         message.AddAction (new MessageAction (String.Format ("Button {0}", i)));
                     }
                 }
-                    
+
                 message.ThawNotify ();
                 PushMessage (message);
-                
+
                 if (count == 3) {
                     message_3 = message;
                 }
-                
+
                 return true;
             });
         }
-        
-        protected virtual void RemoveMessage (SourceMessage message)
+
+        public void RemoveMessage (SourceMessage message)
         {
             lock (this) {
                 if (messages.Remove (message)) {
                     message.Updated -= HandleMessageUpdated;
                     OnMessageNotify ();
                 }
-            }   
+            }
         }
 
         private void HandleMessageUpdated (object o, EventArgs args)
         {
+            if (CurrentMessage == o && CurrentMessage.IsHidden) {
+                PopMessage ();
+            }
+
             OnMessageNotify ();
         }
-        
+
         protected virtual void OnMessageNotify ()
         {
             EventHandler handler = MessageNotify;
@@ -389,9 +461,11 @@ namespace Banshee.Sources
                 handler (this, EventArgs.Empty);
             }
         }
-    
+
         protected virtual void OnChildSourceAdded (Source source)
         {
+            SortChildSources ();
+            source.Updated += OnChildSourceUpdated;
             ThreadAssist.ProxyToMain (delegate {
                 SourceEventHandler handler = ChildSourceAdded;
                 if (handler != null) {
@@ -401,9 +475,10 @@ namespace Banshee.Sources
                 }
             });
         }
-        
+
         protected virtual void OnChildSourceRemoved (Source source)
         {
+            source.Updated -= OnChildSourceUpdated;
             ThreadAssist.ProxyToMain (delegate {
                 SourceEventHandler handler = ChildSourceRemoved;
                 if (handler != null) {
@@ -413,7 +488,7 @@ namespace Banshee.Sources
                 }
             });
         }
-        
+
         protected virtual void OnUpdated ()
         {
             EventHandler handler = Updated;
@@ -422,6 +497,11 @@ namespace Banshee.Sources
             }
         }
 
+        protected virtual void OnChildSourceUpdated (object o, EventArgs args)
+        {
+            SortChildSources ();
+        }
+
         public void NotifyUser ()
         {
             OnUserNotifyUpdated ();
@@ -436,24 +516,24 @@ namespace Banshee.Sources
                 }
             }
         }
-        
+
 #endregion
-        
+
 #region Private Methods
-        
+
         private void OnPropertyChanged (object o, PropertyChangeEventArgs args)
         {
             OnUpdated ();
         }
-        
+
 #endregion
-        
+
 #region Public Properties
-        
+
         public ReadOnlyCollection<Source> Children {
             get { return read_only_children; }
         }
-        
+
         string [] ISource.Children {
             get { return null; }
         }
@@ -461,17 +541,21 @@ namespace Banshee.Sources
         public Source Parent {
             get { return parent; }
         }
-        
+
+        public virtual string TypeName {
+            get { return GetType ().Name; }
+        }
+
         private string unique_id;
         public string UniqueId {
             get {
                 if (unique_id == null && type_unique_id == null) {
-                    Log.ErrorFormat ("Creating Source.UniqueId for {0}, but TypeUniqueId is null; trace is {1}", this.Name, System.Environment.StackTrace);
+                    Log.ErrorFormat ("Creating Source.UniqueId for {0} (type {1}), but TypeUniqueId is null; trace is {2}", this.Name, GetType ().Name, System.Environment.StackTrace);
                 }
-                return unique_id ?? unique_id = String.Format ("{0}-{1}", this.GetType ().Name, TypeUniqueId);
+                return unique_id ?? (unique_id = String.Format ("{0}-{1}", this.GetType ().Name, TypeUniqueId));
             }
         }
-        
+
         private string type_unique_id;
         protected string TypeUniqueId {
             get { return type_unique_id; }
@@ -485,11 +569,11 @@ namespace Banshee.Sources
         public virtual bool HasProperties {
             get { return false; }
         }
-        
+
         public virtual bool HasViewableTrackProperties {
             get { return false; }
         }
-        
+
         public virtual bool HasEditableTrackProperties {
             get { return false; }
         }
@@ -503,12 +587,12 @@ namespace Banshee.Sources
             get { return properties.Get<string> ("GenericName"); }
             set { properties.SetString ("GenericName", value); }
         }
-        
+
         public int Order {
             get { return properties.GetInteger ("Order"); }
             set { properties.SetInteger ("Order", value); }
         }
-        
+
         public SourceMessage CurrentMessage {
             get { lock (this) { return messages.Count > 0 ? messages[0] : null; } }
         }
@@ -516,42 +600,46 @@ namespace Banshee.Sources
         public virtual bool ImplementsCustomSearch {
             get { return false; }
         }
-        
+
         public virtual bool CanSearch {
             get { return false; }
         }
-                
+
         public virtual string FilterQuery {
             get { return properties.Get<string> ("FilterQuery"); }
             set { properties.SetString ("FilterQuery", value); }
         }
-        
+
         public TrackFilterType FilterType {
             get { return (TrackFilterType)properties.GetInteger ("FilterType"); }
             set { properties.SetInteger ("FilterType", (int)value); }
         }
-        
+
         public virtual bool Expanded {
             get { return properties.GetBoolean ("Expanded"); }
             set { properties.SetBoolean ("Expanded", value); }
         }
-        
+
         public virtual bool? AutoExpand {
             get { return true; }
         }
-        
+
         public virtual PropertyStore Properties {
             get { return properties; }
         }
-        
+
         public virtual bool CanActivate {
             get { return true; }
         }
-        
+
         public virtual int Count {
             get { return 0; }
         }
 
+        public virtual int EnabledCount {
+            get { return Count; }
+        }
+
         private string parent_conf_id;
         public string ParentConfigurationId {
             get {
@@ -564,97 +652,154 @@ namespace Banshee.Sources
 
         private string conf_id;
         public string ConfigurationId {
-            get { return conf_id ?? conf_id = UniqueId.Replace ('.', '_'); }
+            get { return conf_id ?? (conf_id = UniqueId.Replace ('.', '_')); }
         }
 
         public virtual int FilteredCount { get { return Count; } }
-                
+
         public virtual string TrackModelPath {
             get { return null; }
         }
-        
+
+        public static readonly SourceSortType SortNameAscending = new SourceSortType (
+            "NameAsc",
+            Catalog.GetString ("Name"),
+            SortType.Ascending, null); // null comparer b/c we already fall back to sorting by name
+
+        public static readonly SourceSortType SortSizeAscending = new SourceSortType (
+            "SizeAsc",
+            Catalog.GetString ("Size Ascending"),
+            SortType.Ascending, new SizeComparer ());
+
+        public static readonly SourceSortType SortSizeDescending = new SourceSortType (
+            "SizeDesc",
+            Catalog.GetString ("Size Descending"),
+            SortType.Descending, new SizeComparer ());
+
+        private static SourceSortType[] sort_types = new SourceSortType[] {};
+        public virtual SourceSortType[] ChildSortTypes {
+            get { return sort_types; }
+        }
+
+        public SourceSortType ActiveChildSort {
+            get { return child_sort; }
+        }
+
+        public virtual SourceSortType DefaultChildSort {
+            get { return null; }
+        }
+
+        public bool SeparateChildrenByType {
+            get { return separate_by_type_schema.Get (); }
+            set {
+                separate_by_type_schema.Set (value);
+                SortChildSources ();
+            }
+        }
+
 #endregion
 
-#region Status Message Stuff        
-        
+#region Status Message Stuff
+
         private static DurationStatusFormatters duration_status_formatters = new DurationStatusFormatters ();
         public static DurationStatusFormatters DurationStatusFormatters {
             get { return duration_status_formatters; }
         }
-        
+
         protected virtual int StatusFormatsCount {
             get { return duration_status_formatters.Count; }
         }
-        
+
         public virtual int CurrentStatusFormat {
             get { return ConfigurationClient.Get<int> (String.Format ("sources.{0}", ParentConfigurationId), "status_format", 0); }
             set { ConfigurationClient.Set<int> (String.Format ("sources.{0}", ParentConfigurationId), "status_format", value); }
         }
-        
+
         public SchemaEntry<T> CreateSchema<T> (string name)
         {
             return CreateSchema<T> (name, default(T), null, null);
         }
-        
+
         public SchemaEntry<T> CreateSchema<T> (string name, T defaultValue, string shortDescription, string longDescription)
         {
-            return new SchemaEntry<T> (String.Format ("sources.{0}", ParentConfigurationId), name, defaultValue, shortDescription, longDescription); 
+            return new SchemaEntry<T> (String.Format ("sources.{0}", ParentConfigurationId), name, defaultValue, shortDescription, longDescription);
         }
-        
+
         public SchemaEntry<T> CreateSchema<T> (string ns, string name, T defaultValue, string shortDescription, string longDescription)
         {
-            return new SchemaEntry<T> (String.Format ("sources.{0}.{1}", ParentConfigurationId, ns), name, defaultValue, shortDescription, longDescription); 
+            return new SchemaEntry<T> (String.Format ("sources.{0}.{1}", ParentConfigurationId, ns), name, defaultValue, shortDescription, longDescription);
         }
-        
+
+        public virtual string PreferencesPageId {
+            get { return null; }
+        }
+
+        private Banshee.Preferences.SourcePage prefs_page;
+        public Banshee.Preferences.Page PreferencesPage {
+            get {
+                return prefs_page ?? (prefs_page = new Banshee.Preferences.SourcePage (this));
+            }
+        }
+
         public void CycleStatusFormat ()
         {
             int new_status_format = CurrentStatusFormat + 1;
             if (new_status_format >= StatusFormatsCount) {
                 new_status_format = 0;
             }
-            
+
             CurrentStatusFormat = new_status_format;
         }
 
+        private const string STATUS_BAR_SEPARATOR = " \u2013 ";
         public virtual string GetStatusText ()
         {
             StringBuilder builder = new StringBuilder ();
 
             int count = FilteredCount;
-            
+
             if (count == 0) {
                 return String.Empty;
             }
-            
-            builder.AppendFormat (Catalog.GetPluralString ("{0} item", "{0} items", count), count);
-            
+
+            var count_str = String.Format ("{0:N0}", count);
+            builder.AppendFormat (GetPluralItemCountString (count), count_str);
+
             if (this is IDurationAggregator && StatusFormatsCount > 0) {
-                builder.Append (", ");
-                duration_status_formatters[CurrentStatusFormat] (builder, ((IDurationAggregator)this).Duration);
+                var duration = ((IDurationAggregator)this).Duration;
+                if (duration > TimeSpan.Zero) {
+                    builder.Append (STATUS_BAR_SEPARATOR);
+                    duration_status_formatters[CurrentStatusFormat] (builder, ((IDurationAggregator)this).Duration);
+                }
             }
 
             if (this is IFileSizeAggregator) {
                 long bytes = (this as IFileSizeAggregator).FileSize;
                 if (bytes > 0) {
-                    builder.Append (", ");
+                    builder.Append (STATUS_BAR_SEPARATOR);
                     builder.AppendFormat (new FileSizeQueryValue (bytes).ToUserQuery ());
                 }
             }
-            
+
             return builder.ToString ();
         }
-        
+
+        public virtual string GetPluralItemCountString (int count)
+        {
+            return Catalog.GetPluralString ("{0} item", "{0} items", count);
+        }
+
 #endregion
 
         public override string ToString ()
         {
             return Name;
         }
-        
+
         /*string IService.ServiceName {
             get { return String.Format ("{0}{1}", DBusServiceManager.MakeDBusSafeString (Name), "Source"); }
         }*/
-        
+
         // FIXME: Replace ISource with IDBusExportable when it's enabled again
         ISource ISource.Parent {
             get {
